// Copyright (c) 2019-2020 The Authors.
// This file is part of @npcz/ngx-search-bar.
//
// SPDX-License-Identifier: BSD-3-Clause
import { __decorate, __metadata } from "tslib";
import { Component, Input, Output, EventEmitter, ViewChild, ElementRef, } from '@angular/core';
import { FormBuilder } from '@angular/forms';
import { MatButton } from '@angular/material/button';
import { MatAutocomplete } from '@angular/material/autocomplete';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
let SearchBarComponent = class SearchBarComponent {
    constructor(_fb) {
        this._fb = _fb;
        this.highlightStyle = {
            hover: 'background',
            focus: 'outline',
        };
        this.search = new EventEmitter();
        this.changes = new EventEmitter();
        this.searchQuery = '';
        this._searchForm = this._fb.group({
            searchInput: null,
        });
        // Automatic unsubscription when the component is destroyed
        this._ngUnsubscribe$ = new Subject();
    }
    get autoComplete() {
        return this._autoComplete;
    }
    set autoComplete(value) {
        this._autoComplete = value;
        this._setupAutoCompleteSelectionHandler();
    }
    ngOnInit() {
        this._searchForm
            .get('searchInput')
            .valueChanges.pipe(takeUntil(this._ngUnsubscribe$))
            .subscribe((value) => {
            this.searchQuery = value;
            console.debug(`emit changes: ${this.category} / ${this.searchQuery}`);
            this.changes.emit({
                category: this.category,
                query: this.searchQuery,
            });
        });
    }
    ngOnDestroy() {
        // The emitted value is not important and not to be used, but is here
        // simply because it does not make sense to call next() with no value...
        this._ngUnsubscribe$.next(true);
        // Security measure to avoid memory leaks, as calling complete on
        // the source stream will remove the references to all the subscribed
        // observers, allowing the garbage collector to eventually dispose any
        // non unsubscribed Subscription instance.
        this._ngUnsubscribe$.complete();
    }
    clear() {
        this._searchForm.get('searchInput').setValue('');
        this._setFocusToSearchInput();
    }
    enable() {
        this.disabled = false;
    }
    disable() {
        this.clear();
        this.disabled = true;
    }
    focus() {
        this._setFocusToSearchInput();
    }
    _onCategorySelected(value) {
        this.category = value;
        console.debug(`emit changes: ${this.category} / ${this.searchQuery}`);
        this.changes.emit({
            category: this.category,
            query: this.searchQuery,
        });
    }
    _onCategoryMenuClosed() {
        this._setFocusToSearchInput();
    }
    _onSubmit() {
        console.debug(`submit: ${this.category} / ${this.searchQuery}`);
        this.search.emit({
            category: this.category,
            query: this.searchQuery,
        });
        this._searchInput.nativeElement.blur();
    }
    _setupAutoCompleteSelectionHandler() {
        this._autoComplete.optionSelected
            .pipe(takeUntil(this._ngUnsubscribe$))
            .subscribe(() => {
            // Trigger a form submit by clicking the submit button
            this._submitButton._elementRef.nativeElement.click();
        });
    }
    _setFocusToSearchInput() {
        this._searchInput.nativeElement.focus();
        // Move the cursor to the end of the text input value so that the user is
        // ready to type
        this._moveCursorToEnd();
    }
    _moveCursorToEnd() {
        const el = this._searchInput.nativeElement;
        if (typeof el.selectionStart == 'number') {
            el.selectionStart = el.selectionEnd = el.value.length;
        }
        else if (typeof el.createTextRange != 'undefined') {
            const range = el.createTextRange();
            range.collapse(false);
            range.select();
        }
    }
};
SearchBarComponent.ctorParameters = () => [
    { type: FormBuilder }
];
__decorate([
    ViewChild('search__input'),
    __metadata("design:type", ElementRef)
], SearchBarComponent.prototype, "_searchInput", void 0);
__decorate([
    ViewChild('search__submit'),
    __metadata("design:type", MatButton)
], SearchBarComponent.prototype, "_submitButton", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], SearchBarComponent.prototype, "placeholder", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], SearchBarComponent.prototype, "clearButton", void 0);
__decorate([
    Input(),
    __metadata("design:type", Array)
], SearchBarComponent.prototype, "categoryList", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], SearchBarComponent.prototype, "category", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], SearchBarComponent.prototype, "disabled", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SearchBarComponent.prototype, "highlightStyle", void 0);
__decorate([
    Input(),
    __metadata("design:type", MatAutocomplete),
    __metadata("design:paramtypes", [MatAutocomplete])
], SearchBarComponent.prototype, "autoComplete", null);
__decorate([
    Output(),
    __metadata("design:type", Object)
], SearchBarComponent.prototype, "search", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], SearchBarComponent.prototype, "changes", void 0);
SearchBarComponent = __decorate([
    Component({
        selector: 'hc-search-bar',
        template: "<form\n  [formGroup]=\"_searchForm\"\n  id=\"search__wrapper\"\n  class=\"search__wrapper\"\n  [hcHighlight]=\"highlightStyle\"\n  (ngSubmit)=\"_onSubmit()\"\n>\n  <div class=\"search__container\" fxLayout=\"row\">\n    <!--\n      Set the type to 'button' so that the only submit button in the form is\n      the 'search__submit' button\n    -->\n    <button\n      type=\"button\"\n      *ngIf=\"categoryList\"\n      mat-stroked-button\n      class=\"search-category__button\"\n      [disabled]=\"disabled\"\n      [matMenuTriggerFor]=\"appMenu\"\n      (menuClosed)=\"_onCategoryMenuClosed()\"\n    >\n      <div fxLayout=\"row\" fxFlex=\"grow\">\n        <div\n          class=\"search-category__text\"\n          fxFlex=\"grow\"\n          ellipsis\n          [ellipsis-content]=\"category\"\n        ></div>\n        <mat-icon fxFlexAlign=\"center\">arrow_drop_down</mat-icon>\n      </div>\n    </button>\n\n    <!--\n      Because autocomplete is optional, we avoid errors on the input field by\n      setting the matAutocompleteDisabled property based on whether we do have\n      an autoComplete component set or not.\n    -->\n    <input\n      #search__input\n      type=\"text\"\n      matInput\n      class=\"search__input\"\n      fxFlex=\"grow\"\n      [placeholder]=\"placeholder\"\n      formControlName=\"searchInput\"\n      [matAutocomplete]=\"autoComplete\"\n      [matAutocompleteDisabled]=\"!autoComplete\"\n    />\n    <div *ngIf=\"searchQuery && clearButton\" fxLayout=\"row\">\n      <!--\n        Set the type to 'button' so that the only submit button in the form is\n        the 'search__submit' button\n      -->\n      <button\n        type=\"button\"\n        mat-icon-button\n        aria-label=\"Clear\"\n        (click)=\"clear()\"\n      >\n        <mat-icon>close</mat-icon>\n      </button>\n      <div fxFlexAlign=\"center\" class=\"clear__separator\"></div>\n    </div>\n    <!-- This is the only button that must be set with type 'submit' -->\n    <button #search__submit type=\"submit\" mat-icon-button [disabled]=\"disabled\">\n      <mat-icon>search</mat-icon>\n    </button>\n  </div>\n</form>\n\n<mat-menu #appMenu=\"matMenu\">\n  <ng-template matMenuContent>\n    <button\n      *ngFor=\"let category of categoryList\"\n      (click)=\"_onCategorySelected(category)\"\n      mat-menu-item\n    >\n      {{ category }}\n    </button>\n  </ng-template>\n</mat-menu>\n",
        styles: [".clear__separator{border-right:thin solid;border-color:#a9a9a9;width:5px;height:60%;margin-right:5px}.search__input{border:none;outline:0;margin:0}.search__wrapper{margin:4px;padding:5px;border-radius:8px}.search-category__button{padding:0 0 0 5px;margin-right:5px;min-width:80px;max-width:160px;flex:1 0 20%}.search-category__text{overflow:hidden}"]
    }),
    __metadata("design:paramtypes", [FormBuilder])
], SearchBarComponent);
export { SearchBarComponent };
//# sourceMappingURL=search-bar.component.js.map